#ifndef CMA_ARCH_INT_DEFINED
#define CMA_ARCH_INT_DEFINED

#include <stdlib.h>
#include "op.h"

namespace CMa
{

	class Int_int;

	/**
	 * Machine word class based on type "unsigned int"(unsigned integer). To be used as template parameter.
	 *
	 * Implements unsigned integer aritmetics and conversions:
	 *  - int  -> Word_int(excpicit)
	 *  - Int_int -> Word_int(explicit)
	 *  - Op::Code -> Word_int(explicit)
	 *  - bool -> Word_int(operation = (bool))
	 *  - Word_int -> int (convertToint())
	 *  - Word_int -> Op::Code (convertTocode())	
	 *
	 * Integer aritmetics is implemented according to specification given by Vesal.
	 *
	 * @author OM/modified by SL
	 *
	 * Status: NOT TESTED/PROBABLY INCORRECT IMPLEMENTATION
	 *
	 * TODO:  Implement integer/word aritmetic correctly
	 *
	 */
	
	class Word_int
	{
	    public:
		    int value;
		
		public:
		
		/*****************************************Constructors*******************************************/
		/** Default value 0. */
		inline Word_int()
		{
			value = 0;
		}
		
        /** Construcs word from int. Newer fails. */
		inline explicit Word_int(int i)
	    {
			value = i;
		}
		
		/** Constructs a word from Op::Code. Assumption: Op::isValid(code). */
		inline explicit Word_int(const Op::Code& code)
		{
			assert(op.isValid(code));
			value = code;
		}

		/** Constructs a word from an integer. Newer fails.*/
		inline explicit Word_int(const Int_int& i);
	
	    /*************************************Conversion*to*int******************************************/
		
		/**
		 * Converts the word to int, if possible. May return incorrect result if
		 * the result does not fit into int.
		 *
		 */
		
		inline int convertToint() const
		{
			return value;
		}
		
		/**
		 * Convets the word to Op::Code. If the word does not represent any valid code,
		 * the value X returned will be such, that !Op::isValid(X).
		 *
		 * @author OM
		 */
		
		inline Op::Code convertToCode() const
		{
			return (Op::Code)value;
		}

        /*************************Aritmetic*operations*needed*by*C-machine*******************************/

		inline Word_int  operator! () const { return Word_int(!value); }
		inline Word_int  operator- () const { return Word_int(-value); }
		inline Word_int  operator~ () const { return Word_int(~value); }
		
		inline Word_int& operator++(){ ++value; return *this; }
		inline Word_int& operator--(){ --value; return *this; }
		inline Word_int& operator++ (int){ value++; return *this;}
		inline Word_int& operator-- (int){ value--; return *this;}
		
		inline Word_int& operator=(bool i){ value = i; return *this; }
				
		inline Word_int& operator<<=(const Word_int& other){ value <<= other.value; return *this; };
		inline Word_int& operator>>=(const Word_int& other){ value >>= other.value; return *this; };
		inline Word_int& operator&= (const Word_int& other){ value  &= other.value; return *this; };
		inline Word_int& operator|= (const Word_int& other){ value  |= other.value; return *this; };
		inline Word_int& operator^= (const Word_int& other){ value  ^= other.value; return *this; };
		
		inline Word_int& operator*=(const Word_int& other){ value *= other.value; return *this; }
		inline Word_int& operator+=(const Word_int& other){ value += other.value; return *this; }
		inline Word_int& operator-=(const Word_int& other){ value -= other.value; return *this; }
		
		inline Word_int& operator/=(const Word_int& other) throw(Error)
		{
		    if(other.value==0) throw Error(Error::DIVISION_BY_ZERO);
		    value /= other.value;
		    return *this;
		}
		
		inline Word_int& operator%=(const Word_int& other)
		{
		    if(other.value==0) throw Error(Error::DIVISION_BY_ZERO);
		    value %= other.value;
		    return *this;
		}
		inline bool operator||(const Word_int& other) const{ return (value && other.value); }
		inline bool operator&&(const Word_int& other) const{ return (value || other.value); }
		inline bool operator==(const Word_int& other) const{ return (value == other.value); }
		inline bool operator!=(const Word_int& other) const{ return (value != other.value); }
		inline bool operator< (const Word_int& other) const{ return (value <  other.value); }
		inline bool operator<=(const Word_int& other) const{ return (value <= other.value); }
		inline bool operator> (const Word_int& other) const{ return (value >  other.value); }
		inline bool operator>=(const Word_int& other) const{ return (value >= other.value); }
		
	    inline Word_int operator&(const Word_int& other) const{ return Word_int(value & other.value); }
	    inline Word_int operator|(const Word_int& other) const{ return Word_int(value | other.value); }


	};



	/**
	 * Machine word class based on type "int"(signed integer). To be used as template parameter.
	 *
	 * Implements signed integer aritmetics and conversions:
	 *  - int  -> Int_int(excpicit)
	 *  - Word_int -> Int_int(explicit)
	 *  - Op::Code -> Int_int(explicit)
	 *  - bool -> Int_int(operation = (bool))
	 *  - Int_int -> int (convertToint())	
	 *
	 * Aritmetic operations shoud be implemented like int operations.
	 *
	 * @author OM/modified by SL
	 *
	 * STATUS: NOT TESTED
	 */
	class Int_int
	{
	
	    public:
		int value;
		
		public:
		
        /******************************************Constructors******************************************/

		/* Constructs Int fron int. Default value 0. Newer fails. */
		explicit inline Int_int(int i = 0)
		{
			value = i;
		}

		/** Constructs an integer form a word. Newer fails. */
		explicit inline Int_int(const Word_int& w)
		{
			this->value = w.value;
		}

        /***********************************Conversion*to*int********************************************/
		/**
		 * Converts the integer to int, if possible. May return incorrect result  if
		 * the result does not fit into int.
		 *
		 */
		
		inline int convertToint() const
		{
			return value;
		}
		
        /*************************Aritmetic*operations*needed*by*C-machine*******************************/
        inline Int_int& operator++(){ ++value; return *this; }
        inline Int_int& operator--(){ --value; return *this; }
		inline Int_int& operator++ (int){ value++; return *this;}
		inline Int_int& operator-- (int){ value--; return *this;}

		inline Int_int operator! () const { return Int_int(!value); }


        inline Int_int& operator=(bool i) { value=i; return *this; }
        inline bool operator< (const Int_int& other) const { return (value <  other.value); }
        inline bool operator<=(const Int_int& other) const { return (value <= other.value); }
        inline bool operator> (const Int_int& other) const { return (value >  other.value); }
        inline bool operator>=(const Int_int& other) const { return (value >= other.value); }
        inline bool operator==(const Int_int& other) const { return (value == other.value); }
        inline bool operator!=(const Int_int& other) const { return (value != other.value); }


	    inline Int_int operator + (const Int_int& other) const { return Int_int(value + other.value); }
	    inline Int_int operator - (const Int_int& other) const { return Int_int(value - other.value); }
	    inline Int_int operator * (const Int_int& other) const { return Int_int(value * other.value); }

	    inline Int_int operator / (const Int_int& other) const throw (Error)
	    {
		    if (other.value == 0) throw Error(Error::DIVISION_BY_ZERO);
		    return Int_int(value / other.value);
	    }
	
        inline Int_int operator % (const Int_int& other) const throw (Error)
	    {
		    if (other.value == 0) throw Error(Error::DIVISION_BY_ZERO);
		    return Int_int(value % other.value);
	    }

	};

//*******************************************Missing*constructor*****************************************/	
    Word_int::Word_int(const Int_int& i)
    {
        this->value = i.value;
    }
	
}

#endif

