#ifndef CMA_FREE_STORE_DEFINED
#define CMA_FREE_STORE_DEFINED

#include <stdlib.h>

namespace CMa
{

	/**
	 * A parametrized implementation of CMa free store (upper part of heap) manager.
	 * Currently just decreases NP on memory requests and does nothing when
	 * memory is freed.
	 *
	 * @author OM
	 *
	 * STATUS: NOT TESTED
	 *
	 * TODO: Implement garbage collector.
	 */
	template<class Word, class Int, class Heap> class FreeStore
	{
	protected:
		Heap& heap;

	public:

		FreeStore(Heap& heap) : heap(heap)
		{}

		
		
		/* Interfaces used by the templates */

		/**
		 * Initializes internal data structures.
		 *
		 * @author OM
		 */
		void init()
		{
			// Nothing for now
		}

		/**
		 * Verifies if the free store is in correct state when operation HALT is executed.
		 *
		 * @throw Error if free store is in incorrect state
		 *
		 * @author OM
		 */
		void verifyHalt() throw (Error)
		{
			// Nothing for now
		}

		void deInit()
		{
			// Nothing for now
		}

		/**
		 * Allocates an area of memory consisting of "size" words.
		 *
		 * Assumption: size > 0
		 *
		 * @return address of newly allocated area or 0, if unsuccessful (because of either NP - size < EP):
		 *
		 * @author OM/ modified by SL
		 */
		inline Word malloc(Int size)
		{
			assert(size > (Int)0);

			Int newNP = heap.getNP() - size;
			if (!heap.verifyNP(newNP)) return (Word)0;
            else
            {
                heap.setNP(newNP);
                return (Word)newNP;
            }
			
		}

		/**
		 * Releases a chunk of memory that was allocated at the given address.
		 *
		 * @throw Error if the address passed is incorrect
		 *
		 * @author OM
		 */
		inline void free(Int addr) throw (Error)
		{
			// Nothing for now
		}
	};
}

#endif

